// SPDX-License-Identifier: AGPL-3.0-or-later, OR GPL-2.0-or-later WITH linking exception
// SPDX-FileCopyrightText: 2020 grammm GmbH
// This file is part of Gromox.
#include <stddef.h>
#include <libHX/defs.h>
#include "rop_ids.h"

#define E(s) [s] = #s
static const char *const rop_names[] = {
	E(ropRelease),
	E(ropOpenFolder),
	E(ropOpenMessage),
	E(ropGetHierarchyTable),
	E(ropGetContentsTable),
	E(ropCreateMessage),
	E(ropGetPropertiesSpecific),
	E(ropGetPropertiesAll),
	E(ropGetPropertiesList),
	E(ropSetProperties),
	E(ropDeleteProperties),
	E(ropSaveChangesMessage),
	E(ropRemoveAllRecipients),
	E(ropModifyRecipients),
	E(ropReadRecipients),
	E(ropReloadCachedInformation),
	E(ropSetMessageReadFlag),
	E(ropSetColumns),
	E(ropSortTable),
	E(ropRestrict),
	E(ropQueryRows),
	E(ropGetStatus),
	E(ropQueryPosition),
	E(ropSeekRow),
	E(ropSeekRowBookmark),
	E(ropSeekRowFractional),
	E(ropCreateBookmark),
	E(ropCreateFolder),
	E(ropDeleteFolder),
	E(ropDeleteMessages),
	E(ropGetMessageStatus),
	E(ropSetMessageStatus),
	E(ropGetAttachmentTable),
	E(ropOpenAttachment),
	E(ropCreateAttachment),
	E(ropDeleteAttachment),
	E(ropSaveChangesAttachment),
	E(ropSetReceiveFolder),
	E(ropGetReceiveFolder),
	E(ropRegisterNotification),
	E(ropRegisterNotify),
	E(ropOpenStream),
	E(ropReadStream),
	E(ropWriteStream),
	E(ropSeekStream),
	E(ropSetStreamSize),
	E(ropSetSearchCriteria),
	E(ropGetSearchCriteria),
	E(ropSubmitMessage),
	E(ropMoveCopyMessages),
	E(ropAbortSubmit),
	E(ropMoveFolder),
	E(ropCopyFolder),
	E(ropQueryColumnsAll),
	E(ropAbort),
	E(ropCopyTo),
	E(ropCopyToStream),
	E(ropCloneStream),
	E(ropGetPermissionsTable),
	E(ropGetRulesTable),
	E(ropModifyPermissions),
	E(ropModifyRules),
	E(ropGetOwningServers),
	E(ropLongTermIdFromId),
	E(ropIdFromLongTermId),
	E(ropPublicFolderIsGhosted),
	E(ropOpenEmbeddedMessage),
	E(ropSetSpooler),
	E(ropSpoolerLockMessage),
	E(ropGetAddressTypes),
	E(ropTransportSend),
	E(ropFastTransferSourceCopyMessages),
	E(ropFastTransferSourceCopyFolder),
	E(ropFastTransferSourceCopyTo),
	E(ropFastTransferSourceGetBuffer),
	E(ropFindRow),
	E(ropProgress),
	E(ropTransportNewMail),
	E(ropGetValidAttachments),
	E(ropFastTransferDestinationConfigure),
	E(ropFastTransferDestinationPutBuffer),
	E(ropGetNamesFromPropertyIds),
	E(ropGetPropertyIdsFromNames),
	E(ropUpdateDeferredActionMessages),
	E(ropEmptyFolder),
	E(ropExpandRow),
	E(ropCollapseRow),
	E(ropLockRegionStream),
	E(ropUnlockRegionStream),
	E(ropCommitStream),
	E(ropGetStreamSize),
	E(ropQueryNamedProperties),
	E(ropGetPerUserLongTermIds),
	E(ropGetPerUserGuid),
	E(ropReadPerUserInformation),
	E(ropWritePerUserInformation),
	E(ropSetReadFlags),
	E(ropCopyProperties),
	E(ropGetReceiveFolderTable),
	E(ropFastTransferSourceCopyProperties),
	E(ropGetCollapseState),
	E(ropSetCollapseState),
	E(ropGetTransportFolder),
	E(ropPending),
	E(ropOptionsData),
	E(ropSynchronizationConfigure),
	E(ropSynchronizationImportMessageChange),
	E(ropSynchronizationImportHierarchyChange),
	E(ropSynchronizationImportDeletes),
	E(ropSynchronizationUploadStateStreamBegin),
	E(ropSynchronizationUploadStateStreamContinue),
	E(ropSynchronizationUploadStateStreamEnd),
	E(ropSynchronizationImportMessageMove),
	E(ropSetPropertiesNoReplicate),
	E(ropDeletePropertiesNoReplicate),
	E(ropGetStoreState),
	E(ropSynchronizationOpenCollector),
	E(ropGetLocalReplicaIds),
	E(ropSynchronizationImportReadStateChanges),
	E(ropResetTable),
	E(ropSynchronizationGetTransferState),
	E(ropTellVersion),
	E(ropFreeBookmark),
	E(ropWriteAndCommitStream),
	E(ropHardDeleteMessages),
	E(ropHardDeleteMessagesAndSubfolders),
	E(ropSetLocalReplicaMidsetDeleted),
	E(ropBackoff),
	E(ropLogon),
	E(ropBufferTooSmall),
};
#undef E

const char *rop_idtoname(unsigned int i)
{
	const char *s = i < ARRAY_SIZE(rop_names) ? rop_names[i] : NULL;
	return s != NULL ? s : "";
}
